#ifndef SHERPA_SHERPA_HH 
#define SHERPA_SHERPA_HH

#include "AGILe/Generator.hh"
#include "Sherpa.H"


namespace AGILe {

  class Sherpa : public Generator {
  public:
    /// Standard constructor.
    Sherpa(); 

    /// Destructor.
    ~Sherpa() { }

    /// Set up initial state from supplied params.
    Status setGenSpecificInitialState(PdgCode p1 = PROTON, double e1 = 7000.0, 
                           PdgCode p2 = PROTON, double e2 = 7000.0);
    
    /// Run the generator for one event.
    Status makeEvent(HepMC::GenEvent& evt);

    /// Ensure that base class setParam is visible.
    using Generator::setParam;
    
    /// Set string-valued parameter.
    Status setParam(const string& name, const string& value) {
      Generator::setParam(name, value);
      _parameters[name] = value;
      return SUCCESS;
    }

    /// Tidy up after ourselves.
    Status finalize();

    /// Set the random number generator seed.
    Status setSeed(const int value);

  protected:
    /// Sherpa generator as data member.
    SHERPA::Sherpa _sherpa;

    /// Set up default params etc.
    Status initialize();

    /// Fill a HepMC event.
    Status fillEvent(HepMC::GenEvent& evt);

    /// Parameter map.
    std::map<std::string, std::string> _parameters;

  };


}

#endif
