#include "AGILe/AlpGen/AlpGenFPythia.hh"
#include "AGILe/AlpGen/AlpGenWrapper.hh"
#include "AGILe/Utils.hh"
#include "HepMC/HEPEVT_Wrapper.h"


//-----------------------------------------------------------------------------
// Implementation file for class : AlpGenFPythia
// This is the file-reading interface only. It needs to be used in conjunction 
// with FHerwig or FPythia. This class is only to provide access to the parameter 
// settings.
//
// Author: Jon Butterworth
//-----------------------------------------------------------------------------


namespace AGILe {

  // Standard constructor
  AlpGenFPythia::AlpGenFPythia() {

    // Set flag to read from LHA event file
    FPythia::setParam("MSTP(143)", 1);

    // Set the ALPGEN filename to "zhjj" by default
    FC_AHIO.FILENAME[0] = 'z';
    FC_AHIO.FILENAME[1] = 'h';
    FC_AHIO.FILENAME[2] = 'j';
    FC_AHIO.FILENAME[3] = 'j';
  }


  // Set string parameters
  Status AlpGenFPythia::setParam(const string& name, const string& value) {
    Generator::setParam(name, value);

    // Strings
    if (name == "ALPGENFILE") {
      cout << "Setting ALPGENFILE = " << value << endl;
      size_t nch = (value.size() < 100) ? value.size() : 99;
      for (size_t i = 0; i < 100; i++) {
        if (i < nch) {
          FC_AHIO.FILENAME[i] = value.data()[i];
        } else {
          FC_AHIO.FILENAME[i] = ' ';
        }
      }
    } 

    // Integers
    if (name == "IEXC") {
      cout << "Setting ALPGEN IEXC = " << value << endl;
      FC_AHOPTS.IEXC = asInt(value);
    }

    // Doubles
    else if (name == "ETACLMAX") { 
      cout << "Setting ETACLMAX = " << value << endl;
      FC_AHOPTS.ETACLMAX = asDouble(value);
    } 

    // Fallback
    else if (FPythia::setParam(name, value) != SUCCESS) {
      cerr << "AlpGenFPythia doesn't have a parameter called " << name << endl;
      return FAILURE;
    }
    return SUCCESS;
  }

}


// Class factory
extern "C" {
  AGILe::Generator* create() { return new AGILe::AlpGenFPythia(); }
  void destroy(AGILe::Generator* gen) { delete gen; }
}
