// -*- C++ -*-
#include "AGILe/FHerwig/FHerwigJimmy.hh"
#include "AGILe/FHerwig/FHerwig.hh"
#include "AGILe/FHerwig/FHerwigWrapper65.hh"
#include "AGILe/FHerwig/JimmyWrapper.hh"
#include "AGILe/Utils.hh"


namespace AGILe {

  // Standard constructor
  FHerwigJimmy::FHerwigJimmy() {
    // NB. FHerwig constructor gets automatically called first.
    FC_JMPARM.JMBUG = 0;
    // Initialise Jimmy common blocks.
    FC_JIMMIN(); 
  }


  // Run generator initialization
  Status FHerwigJimmy::initialize() {
    FHerwig::initialize();
    // Initialise Jimmy.
    if (FC_JMPARM.MSFLAG == 1) {
      FC_JMINIT();
    }
    _initialized = true;
    return SUCCESS;
  }


  // Set parameters.
  Status FHerwigJimmy::setParam(const string& name, const string& value) {
    Generator::setParam(name, value);

    // Integers.
    if (name == "JMUEO") { 
      cout << "Setting JMUEO = " << asInt(value) << endl;
      FC_JMPARM.JMUEO = asInt(value);
    } else if (name == "JMBUG") { 
      cout << "Setting JMBUG = " << asInt(value) << endl;
      FC_JMPARM.JMBUG = asInt(value);
    } else if (name == "MSFLAG") { 
      cout << "Setting MSFLAG = " << asInt(value) << endl;
      FC_JMPARM.MSFLAG = asInt(value);
    }

    // Doubles.
    else if (name == "PTJIM") {
      cout << "Setting minimum PT for secondary scatters (PTJIM) = " << asDouble(value) << endl;
      FC_JMPARM.PTJIM = asDouble(value);
    } else if (name == "PHAD") { 
      cout << "Setting hadronic photon inverse probability (PHAD) = " << asDouble(value) << endl;
      FC_JMPARM.PHAD = asDouble(value);
    } else if (name == "PRRAD") { 
      cout << "Setting 1/(Proton radius)2 (PRRAD) = " << asDouble(value) << endl;
      FC_JMPARM.JMRAD[72] = asDouble(value);
      FC_JMPARM.JMRAD[74] = asDouble(value);
      FC_JMPARM.JMRAD[90] = asDouble(value);
      FC_JMPARM.JMRAD[92] = asDouble(value);
    } else if (name == "PHRAD") { 
      cout << "Setting 1/(Photon radius)2 (PRRAD) = " << asDouble(value) << endl;
      FC_JMPARM.JMRAD[58]  = asDouble(value);
      FC_JMPARM.JMRAD[120] = asDouble(value);
      FC_JMPARM.JMRAD[126] = asDouble(value);
    }

    // Boolean (as int).
    else if (name == "ANOMOFF") { 
      cout << "Setting MI in anomalous photon events (ANOMOFF) = " << asInt(value) << endl;
      FC_JMPARM.ANOMOFF = asInt(value);
    }

    // Fall back to FHerwig.
    else if (FHerwig::setParam(name, value) != SUCCESS) {
      cerr << "FHerwigJimmy doesn't have a string-valued parameter called " << name << endl;
      return FAILURE;
    }
    return SUCCESS;
  }


  // Run the generator for one event
  /// @todo Make sub-process execution conditional.
  Status FHerwigJimmy::makeEvent(HepMC::GenEvent& evt) {
    Generator::makeEvent(evt);
    int counter(0);
    // Repeat the first stages of event generation until multiple
    // scattering succeeds (or isn't run at all).
    while (true) {
      FC_HWUINE();  // Initialize event
      FC_HWEPRO();  // Generate hard subprocess
      FC_HWBGEN();  // Generate parton cascade

      // Call the multiple scattering routine with the abort flag.
      // If abort is returned with value 1 (true), then abandon the 
      // event. This implementation makes sure that the event generation
      // continues if multiple scattering is disabled (MSPARM != 1)
      int abort(0);
      if (FC_JMPARM.MSFLAG == 1) FC_HWMSCT(&abort);
      // If MS is okay or disabled, escape the loop and 
      // continue with the decays and hadronisation.
      if (abort == 0) break;

      // If repeating MS step, increment counter and clear the GenEvent.
      if (counter++ >= 100) { return FAILURE; }
      clearEvent(evt);
    }
    FC_HWDHOB();  // Do heavy quark decays
    FC_HWCFOR();  // Do cluster formation
    FC_HWCDEC();  // Do cluster decays
    FC_HWDHAD();  // Do unstable particle decays
    FC_HWDHVY();  // Do heavy flavor decays
    FC_HWMEVT();  // Add soft underlying event
    FC_HWUFNE();  // Finish event
    fillEvent(evt);
    return SUCCESS;
  }


  /// Tidy up, print out run stats, etc.
  Status FHerwigJimmy::finalize() {
    FHerwig::finalize();
    if (FC_JMPARM.MSFLAG == 1) { 
      FC_JMEFIN(); 
    }
    return SUCCESS;
  }

}

// Class factory
extern "C" {
  AGILe::Generator* create() { return new AGILe::FHerwigJimmy(); }
  void destroy(AGILe::Generator* gen) { delete gen; }
}
